/*
 * Lcd.h
 *
 * Created: 17.07.2015 01:07:27
 *  Author: bkausbk
 */ 

#ifndef LCD_H_
#define LCD_H_

#define LCD_RS(x) (x ? (PORTA |= _BV(PA5)) : (PORTA &= ~_BV(PA5)))
#define LCD_RW(x) (x ? (PORTA |= _BV(PA3)) : (PORTA &= ~_BV(PA3)))
#define LCD_E(x) (x ? (PORTA |= _BV(PA4)) : (PORTA &= ~_BV(PA4)))
#define LCD_D4(x) (x ? (PORTB |= _BV(PB5)) : (PORTB &= ~_BV(PB5)))
#define LCD_D5(x) (x ? (PORTB |= _BV(PB6)) : (PORTB &= ~_BV(PB6)))
#define LCD_D6(x) (x ? (PORTA |= _BV(PA7)) : (PORTA &= ~_BV(PA7)))
#define LCD_D7(x) (x ? (PORTA |= _BV(PA6)) : (PORTA &= ~_BV(PA6)))

#define LCD_CMD_CLEAR 0x01
#define LCD_CMD_HOME 0x02
#define LCD_CMD_MODE 0x04
#define LCD_CMD_CONTROL 0x08
#define LCD_CMD_SHIFT 0x10
#define LCD_CMD_FUNCTION 0x20
#define LCD_CMD_CGRAM_ADDRESS 0x40
#define LCD_CMD_DDRAM_ADDRESS 0x80

#define LCD_MODE_INCREMENT 0x02
#define LCD_MODE_DECREMENT 0x00
#define LCD_MODE_SHIFT 0x01

#define LCD_CONTROL_DISPLAY_ON 0x04
#define LCD_CONTROL_DISPLAY_OFF 0x00
#define LCD_CONTROL_CURSOR_ON 0x02
#define LCD_CONTROL_CURSOR_OFF 0x00
#define LCD_CONTROL_BLINK_ON 0x01
#define LCD_CONTROL_BLINK_OFF 0x00

#define LCD_SHIFT_CURSOR_RIGHT 0x04
#define LCD_SHIFT_CURSOR_LEFT 0x00
#define LCD_SHIFT_DISPLAY_LEFT 0x80
#define LCD_SHIFT_DISPLAY_RIGHT 0x84

#define LCD_FUNCTION_8BIT 0x10
#define LCD_FUNCTION_4BIT 0x00
#define LCD_FUNCTION_TWO_LINE 0x08
#define LCD_FUNCTION_ONE_LINE 0x00
#define LCD_FUNCTION_FONT5x10 0x04
#define LCD_FUNCTION_FONT5x8 0x00

#define LCD_ROW_ADDRESS_1 0x00
#define LCD_ROW_ADDRESS_3 0x14
#define LCD_ROW_ADDRESS_2 0x40
#define LCD_ROW_ADDRESS_4 0x54

void LcdGenerateEnableClock() {
	LCD_E(1);
	_delay_us(1);
	LCD_E(0);
	_delay_us(50);
}

void LcdWriteCommand4(uint8_t Command) {
	LCD_D7(Command & 8);
	LCD_D6(Command & 4);
	LCD_D5(Command & 2);
	LCD_D4(Command & 1);
	
	LcdGenerateEnableClock();
}

void LcdWriteCommand(uint8_t Command) {
	LCD_RS(0);
	LCD_RW(0);
	
	LcdWriteCommand4(Command >> 4);
	LcdWriteCommand4(Command);
}


void LcdWriteData4(uint8_t Data) {
	LCD_D7(Data & 8);
	LCD_D6(Data & 4);
	LCD_D5(Data & 2);
	LCD_D4(Data & 1);

	LcdGenerateEnableClock();
}

void LcdWriteData(uint8_t Data) {
	LCD_RS(1);
	LCD_RW(0);
	
	LcdWriteData4(Data >> 4);
	LcdWriteData4(Data);
}

void LcdSetCursor(uint8_t x, uint8_t y) {
	if (x >= 20) x = 19;
	if (y >= 4) y = 3;
	
	uint8_t r[4] = {LCD_ROW_ADDRESS_1, LCD_ROW_ADDRESS_2, LCD_ROW_ADDRESS_3, LCD_ROW_ADDRESS_4};
	
	LcdWriteCommand(LCD_CMD_DDRAM_ADDRESS | (x + r[y]));
}

void LcdWriteText(uint8_t x, uint8_t y, const char* Text) {
	if (x >= 20) x = 19;
	if (y >= 4) y = 3;
	
	uint8_t r[4] = {LCD_ROW_ADDRESS_1, LCD_ROW_ADDRESS_2, LCD_ROW_ADDRESS_3, LCD_ROW_ADDRESS_4};
	
	while (*Text) {
		LcdWriteCommand(LCD_CMD_DDRAM_ADDRESS | (x + r[y]));
		LcdWriteData(*Text);
		x++;
		if (x >= 20) {
			x = 0;
			y++;
			if (y >= 4) {
				y = 0;
			}
		}
		Text++;
	}
}

void LcdInitialize() {
	DDRA |= _BV(DDA3) | _BV(DDA4) | _BV(DDA5) | _BV(DDA6) | _BV(DDA7);
	DDRB |= _BV(DDB5) | _BV(DDB6);

	_delay_ms(100);
	
	LCD_RS(0);
	LCD_RW(0);

	LcdWriteCommand4(0x03);
	_delay_us(4500);

	LcdWriteCommand4(0x03);
	//_delay_us(150);
	_delay_us(4500);
	
	LcdWriteCommand4(0x03);
	_delay_us(4500);
	
	LcdWriteCommand4(0x02);

	LcdWriteCommand(LCD_CMD_FUNCTION | LCD_FUNCTION_4BIT | LCD_FUNCTION_TWO_LINE | LCD_FUNCTION_FONT5x8);
	
	LcdWriteCommand(LCD_CMD_CLEAR);
	_delay_us(2000);
	LcdWriteCommand(LCD_CMD_HOME);
	_delay_us(2000);
	
	LcdWriteCommand(LCD_CMD_CONTROL | LCD_CONTROL_DISPLAY_ON | LCD_CONTROL_CURSOR_OFF | LCD_CONTROL_BLINK_ON);
	LcdWriteCommand(LCD_CMD_MODE | LCD_MODE_INCREMENT);

	//    for (unsigned int i = 0x10*2; i < 0x10*2+80; i++) {
	//        LcdWriteData(i);
	//    }
}



#endif /* LCD_H_ */