#pragma once

#define INA226_REGISTER_SHUNTVOLTAGE 0x01
#define INA226_REGISTER_BUSVOLTAGE 0x02
#define INA226_REGISTER_CALIBRATION 0x05
#define INA226_REGISTER_CURRENT 0x04

int16_t Ina226ReadShuntVoltage(const unsigned char Address) {
	i2c_start_wait(Address | I2C_WRITE);
	i2c_write(INA226_REGISTER_SHUNTVOLTAGE);

	i2c_rep_start(Address | I2C_READ);
	unsigned char msb = i2c_readAck();
	unsigned char lsb = i2c_readNak();
	i2c_stop();

	return (int16_t) ((uint16_t) msb << 8 | (uint16_t) lsb);
}

int16_t Ina226ReadCurrent(uint8_t Address) {
	i2c_start_wait(Address | I2C_WRITE);
	i2c_write(INA226_REGISTER_CURRENT);

	i2c_rep_start(Address | I2C_READ);
	unsigned char msb = i2c_readAck();
	unsigned char lsb = i2c_readNak();
	i2c_stop();

	return (int16_t) ((uint16_t) msb << 8 | (uint16_t) lsb);
}

uint16_t Ina226ReadBusVoltage(const unsigned char Address) {
	i2c_start_wait(Address | I2C_WRITE);
	i2c_write(INA226_REGISTER_BUSVOLTAGE);

	i2c_rep_start(Address | I2C_READ);
	unsigned char msb = i2c_readAck();
	unsigned char lsb = i2c_readNak();
	i2c_stop();

	return ((uint16_t) msb << 8 | (uint16_t) lsb);
}

void Ina226SetCalibrationRegister(const uint8_t Address, uint16_t Cal) {
	i2c_start_wait(Address | I2C_WRITE);
	i2c_write(INA226_REGISTER_CALIBRATION);
	i2c_write(Cal >> 8);
	i2c_write(Cal & 0xff);
	i2c_stop();
}

uint16_t Ina226ReadId(const unsigned char Address) {
	i2c_start_wait(Address | I2C_WRITE);
	i2c_write(0xff);

	i2c_rep_start(Address | I2C_READ);
	unsigned char msb = i2c_readAck();
	unsigned char lsb = i2c_readNak();
	i2c_stop();

	return ((uint16_t) msb << 8 | (uint16_t) lsb);
}
